/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/


#include <WaveGen.h>
#include <project.h>
//#include <sine10.h> // 10-bit signed sine table

//static uint8 * pBuffer = WaveGen_Buffer;  // pointer to RAM buffer


//==============================================================================
// Configuration: RAM Buffer -> DMA1 -> VDAC1, DMA2 -> VDAC2, DMA3 -> VDAC3
//==============================================================================

void WaveGen_DMAConfig(void) // 
{
    // Defines for DMA 
    #define TRANSFER_COUNT        (WaveGen_NumSamples)
    #define DMA_BYTES_PER_BURST   1
    #define DMA_REQUEST_PER_BURST 1
    #define DMA_SRC_BASE          (CYDEV_SRAM_BASE) // 
    #define DMA_DST_BASE          (CYDEV_PERIPH_BASE)

    // Configuration for DMA1 
    uint8_t DMA1_Chan;
    uint8_t DMA1_TD[1];
    DMA1_Chan = DMA1_DmaInitialize(DMA_BYTES_PER_BURST, DMA_REQUEST_PER_BURST, HI16(DMA_SRC_BASE), HI16(DMA_DST_BASE));
    DMA1_TD[0] = CyDmaTdAllocate();
    
    CyDmaTdSetConfiguration(DMA1_TD[0], TRANSFER_COUNT, DMA1_TD[0], DMA1__TD_TERMOUT_EN | TD_INC_SRC_ADR);
    CyDmaTdSetAddress(DMA1_TD[0], LO16((uint32)&WaveGen_Buffer[0]), LO16((uint32)VDAC_1_Data_PTR)); // RAM->VDAC1
    CyDmaChSetInitialTd(DMA1_Chan, DMA1_TD[0]);
    
    
    // Configuration for DMA2 
    uint8_t DMA2_Chan;
    uint8_t DMA2_TD[1];
    DMA2_Chan = DMA2_DmaInitialize(DMA_BYTES_PER_BURST, DMA_REQUEST_PER_BURST, HI16(DMA_SRC_BASE), HI16(DMA_DST_BASE));
    DMA2_TD[0] = CyDmaTdAllocate();
    
    CyDmaTdSetConfiguration(DMA2_TD[0], TRANSFER_COUNT, DMA2_TD[0], DMA2__TD_TERMOUT_EN | TD_INC_SRC_ADR);
    CyDmaTdSetAddress(DMA2_TD[0], LO16((uint32)&WaveGen_Buffer[0]+WaveGen_Phase120), LO16((uint32)VDAC_2_Data_PTR)); // RAM->VDAC2
    CyDmaChSetInitialTd(DMA2_Chan, DMA2_TD[0]);
    
    // Configuration for DMA3 
    uint8_t DMA3_Chan;
    uint8_t DMA3_TD[1];
    DMA3_Chan = DMA3_DmaInitialize(DMA_BYTES_PER_BURST, DMA_REQUEST_PER_BURST, HI16(DMA_SRC_BASE), HI16(DMA_DST_BASE));
    DMA3_TD[0] = CyDmaTdAllocate();
    
    CyDmaTdSetConfiguration(DMA3_TD[0], TRANSFER_COUNT, DMA3_TD[0], DMA3__TD_TERMOUT_EN | TD_INC_SRC_ADR);
    CyDmaTdSetAddress(DMA3_TD[0], LO16((uint32)&WaveGen_Buffer[0]+WaveGen_Phase240), LO16((uint32)VDAC_3_Data_PTR)); // RAM->VDAC3
    CyDmaChSetInitialTd(DMA3_Chan, DMA3_TD[0]);
 
    CyDmaChEnable(DMA1_Chan, 1);    // Enable DMA channel
    CyDmaChEnable(DMA2_Chan, 1);    // Enable DMA channel
    CyDmaChEnable(DMA3_Chan, 1);    // Enable DMA channel 
}


//==============================================================================
// Initialize and start component
//==============================================================================

void WaveGen_Start(void) 
{
    WaveGen_CreateWave();   // fill Buffer with default values 
    WaveGen_DMAConfig();    // configure DMA channels
}


//=============================================================================
// change wave amplitude and phase
// make 8-bit sine wave of variable amplitude 0..255 (p-p) centered around 128u
//=============================================================================

//void WaveGen_CreateWave(uint8 ampl, uint8 phase)
void WaveGen_CreateWave()
{
    //const uint8  phase_crrct = 2;   // correction to account for 1 DMA step
    //uint8 v;
    int v;
    uint16 i;
    
    //phase += phase_crrct;           // add phase correction to yield for 2 DMA steps
    /*
    for (i=0; i<WaveGen_NumSamples; i++)
    {
        //uint16 theta = (i + phase) * ((double) MYSINE_TABLE_SIZE/(double) WaveGen_NumSamples); // sine table size 0..1023 scaled to 0..255
        
        // works only for NumSamples=32
        //uint16 theta = (i + phase) << 5; // scale=MYSINE_TABLE_SIZE/NumSamples=1024/32=2^5  
        //uint16 theta = (uint16) (0.5 + (float) (MYSINE_TABLE_SIZE-1) * (float)(i + phase) / (float) (WaveGen_NumSamples) ); // scale=MYSINE_TABLE_SIZE/NumSamples=1024/32=2^5  
        uint16 theta = (uint16) (0.0 + (float) (MYSINE_TABLE_SIZE-1) * ((float)i + 0.5) / (float) (WaveGen_NumSamples) ); // scale=MYSINE_TABLE_SIZE/NumSamples=1024/32=2^5  

        //v = theta>>2; //
        //v = 128 + ( (ampl * mysine(theta)) / 1023.0 / 2 ); // ampl=uint8 [0..255];
        v = 128 + (float)(0.5+ (0.5 * ampl * mysine(theta)) / 1023); // ampl=uint8 [0..255];
        WaveGen_Buffer[i] = v; // ampl=uint8 [0..255];
        WaveGen_Buffer[WaveGen_NumSamples+i] = v; // copy

    } 
    */
    
    
    // todo: calculate 1/4 of sine for speed
    for (i=0; i<WaveGen_NumSamples; i++)
    {
        WaveTable_1_GetSample(i); //todo: missed 1-st sample [0]-first sample
        v = WaveTable_1_sin; // ampl=uint8 [0..255];
        //v=i;//ramp test
        
        // optional check bounds->
        //v = (v > 127)? 127 : (v < -128)? -128 : v; // trim saturate to int8 range
        
        // int32 DDS output, int8/uint8 Buffer->
        v = (uint8) (v^0x80); // convert to uint8 with offset 128
        
        WaveGen_Buffer[i] = v; // ampl=uint8 [0..255];
        WaveGen_Buffer[WaveGen_NumSamples+i] = v; // copy
    } 
    
    
    /*
    for (i=0; i<WaveGen_NumSamples; i++)
    {
        WaveTable_1_GetSample();
        v = WaveTable_1_sin; // ampl=uint8 [0..255];
        //v = (v > 127)? 127 : (v < -128)? -128 : v; // trim saturate to int8 range
        
        // int32 DDS output, int8/uint8 Buffer->
        //v = (uint8) (v^0x80); // convert [-128, +127] to [0, 255] (offset 128)

        v>>=1;
        v+=64;

        //v = (v > 254)? 254 : (v < 0)? 0 : v; // trim saturate to 0-254 range - glitch
        v = (v > 128)? 128 : (v < 0)? 0 : v; // trim saturate to 0-254 range - glitch
        
        WaveGen_Buffer[i] = v; // ampl=uint8 [0..255];
        WaveGen_Buffer[i+WaveGen_NumSamples] = v; // copy
    } 
    */
    
    WaveTable_1_Reset(); // reset to start
}                    





/* [] END OF FILE */
